import logging
from typing import Any

from aporia.core.graphql_client import GraphQLClient
from aporia.core.logging_utils import LOGGER_NAME


logger = logging.getLogger(LOGGER_NAME)


async def log_json(
    graphql_client: GraphQLClient, model_id: str, model_version: str, environment: str, data: Any
):
    """Reports arbitrary data.

    Args:
        graphql_client (GraphQLClient): GraphQL client
        model_id (str): Model ID
        model_version (str): Model version
        environment (str): Environment in which aporia SDK is running
        data (Any): Data to report
    """
    query = """
    mutation LogArbitraryData(
        $modelId: String!,
        $modelVersion: String!,
        $environment: String!,
        $data: GenericScalar!
    ) {
        logArbitraryData(
            modelId: $modelId,
            modelVersion: $modelVersion,
            environment: $environment,
            data: $data
        ) {
            warnings
        }
    }
    """

    variables = {
        "modelId": model_id,
        "modelVersion": model_version,
        "environment": environment,
        "data": data,
    }

    result = await graphql_client.query_with_retries(query, variables)
    for warning in result["logArbitraryData"]["warnings"]:
        logger.warning(warning)



import logging
from typing import List, Optional, Union

import numpy as np
from pandas import Series

from aporia.core.errors import AporiaError
from aporia.core.graphql_client import GraphQLClient
from aporia.core.logging_utils import LOGGER_NAME
from aporia.core.types.field import FieldType

logger = logging.getLogger(LOGGER_NAME)

MAX_BINS = 50


class FieldTrainingData:
    """Field training data."""

    def __init__(
        self,
        field_name: str,
        bins: Optional[List[Union[float, int, str, bool]]] = None,
        counts: Optional[List[int]] = None,
        min: Optional[float] = None,
        max: Optional[float] = None,
        sum: Optional[float] = None,
        median: Optional[float] = None,
        average: Optional[float] = None,
        std: Optional[float] = None,
        variance: Optional[float] = None,
        num_samples: Optional[int] = None,
        num_missing_values: Optional[int] = None,
        num_posinf_values: Optional[int] = None,
        num_neginf_values: Optional[int] = None,
        num_unique_values: Optional[int] = None,
        num_zero_values: Optional[int] = None,
    ):
        """Initializes a FieldTrainingData object.

        Args:
            field_name: Field name
            bins: Histogram bin edges. Defaults to None.
            counts: Hitsogram. Defaults to None.
            min: Minimum value. Defaults to None.
            max: Maximum Value. Defaults to None.
            sum: Sum of all values. Defaults to None.
            median: Median values. Defaults to None.
            average: Average value. Defaults to None.
            std: Standard deviation value. Defaults to None.
            variance: Variance value. Defaults to None.
            num_samples: Number of data samples. Defaults to None.
            num_missing_values: Number of missing values. Defaults to None.
            num_posinf_values: Number of positive infinite values. Defaults to None.
            num_neginf_values: Number of negative infinite values. Defaults to None.
            num_unique_values: Number of unique values. Defaults to None.
            num_zero_values: Number of zero values. Defaults to None.
        """
        self.field_name = field_name
        self.bins = bins
        self.counts = counts
        self.min = min
        self.max = max
        self.sum = sum
        self.median = median
        self.average = average
        self.std = std
        self.variance = variance
        self.num_samples = num_samples
        self.num_missing_values = num_missing_values
        self.num_posinf_values = num_posinf_values
        self.num_neginf_values = num_neginf_values
        self.num_unique_values = num_unique_values
        self.num_zero_values = num_zero_values

    def serialize(self) -> dict:
        """Serializes the field training data to a dict.

        Returns:
            Serialized training data.
        """
        return {
            "fieldName": self.field_name,
            "bins": self.bins,
            "counts": self.counts,
            "min": self.min,
            "max": self.max,
            "sum": self.sum,
            "median": self.median,
            "average": self.average,
            "std": self.std,
            "variance": self.variance,
            "numSamples": self.num_samples,
            "numMissingValues": self.num_missing_values,
            "numPosinfValues": self.num_posinf_values,
            "numNeginfValues": self.num_neginf_values,
            "numUniqueValues": self.num_unique_values,
            "numZeroValues": self.num_zero_values,
        }


async def log_training_data(
    graphql_client: GraphQLClient,
    model_id: str,
    model_version: str,
    features: List[FieldTrainingData],
    labels: List[FieldTrainingData],
    raw_inputs: Optional[List[FieldTrainingData]] = None,
):
    """Reports training data.

    Args:
        graphql_client: GraphQL client
        model_id: Model ID
        model_version: Mode version
        features: Training set features.
        labels: Training set labels.
        raw_inputs: Training set raw inputs. Defaults to None.
    """
    query = """
        mutation LogTrainingSet(
            $modelId: String!,
            $modelVersion: String!,
            $features: [FieldTrainingData]!
            $labels: [FieldTrainingData]!
            $rawInputs: [FieldTrainingData]
        ) {
            logTrainingSet(
                modelId: $modelId,
                modelVersion: $modelVersion,
                features: $features
                labels: $labels
                rawInputs: $rawInputs
            ) {
                warnings
            }
        }
    """

    serialized_raw_inputs = None
    if raw_inputs is not None:
        serialized_raw_inputs = [field_data.serialize() for field_data in raw_inputs]

    variables = {
        "modelId": model_id,
        "modelVersion": model_version,
        "features": [field_data.serialize() for field_data in features],
        "labels": [field_data.serialize() for field_data in labels],
        "rawInputs": serialized_raw_inputs,
    }

    result = await graphql_client.query_with_retries(query, variables)
    for warning in result["logTrainingSet"]["warnings"]:
        logger.warning(warning)


async def log_test_data(
    graphql_client: GraphQLClient,
    model_id: str,
    model_version: str,
    features: List[FieldTrainingData],
    predictions: List[FieldTrainingData],
    labels: List[FieldTrainingData],
    raw_inputs: Optional[List[FieldTrainingData]] = None,
):
    """Reports test data.

    Args:
        graphql_client: GraphQL client
        model_id: Model ID
        model_version: Mode version
        features: Test set features.
        predictions: Test set features.
        labels: Test set labels.
        raw_inputs: Test set raw inputs. Defaults to None.
    """
    query = """
        mutation LogTestSet(
            $modelId: String!,
            $modelVersion: String!,
            $features: [FieldTrainingData]!
            $predictions: [FieldTrainingData]!
            $labels: [FieldTrainingData]!
            $rawInputs: [FieldTrainingData]
        ) {
            logTestSet(
                modelId: $modelId,
                modelVersion: $modelVersion,
                features: $features
                predictions: $predictions
                labels: $labels
                rawInputs: $rawInputs
            ) {
                warnings
            }
        }
    """

    serialized_raw_inputs = None
    if raw_inputs is not None:
        serialized_raw_inputs = [field_data.serialize() for field_data in raw_inputs]

    variables = {
        "modelId": model_id,
        "modelVersion": model_version,
        "features": [field_data.serialize() for field_data in features],
        "predictions": [field_data.serialize() for field_data in predictions],
        "labels": [field_data.serialize() for field_data in labels],
        "rawInputs": serialized_raw_inputs,
    }

    result = await graphql_client.query_with_retries(query, variables)
    for warning in result["logTestSet"]["warnings"]:
        logger.warning(warning)


def calculate_training_data(
    field_name: str, field_data: Series, field_type: FieldType
) -> FieldTrainingData:
    """Calculates training data for a single field.

    Args:
        field_name (str): Field name
        field_data (Series): Field data
        field_type (FieldType): Field type

    Returns:
        FieldTrainingData: Field training data.
    """
    # We currently don't support datetime training data
    if field_type == FieldType.DATETIME:
        return FieldTrainingData(field_name=field_name)
    elif field_type == FieldType.NUMERIC:
        return _calculate_numeric_training_data(field_name=field_name, field_data=field_data)
    elif field_type in [FieldType.BOOLEAN, FieldType.STRING, FieldType.CATEGORICAL]:
        return _calculate_categorical_training_data(
            field_name=field_name, field_data=field_data, field_type=field_type
        )

    raise AporiaError("Unsupported field type {} of field {}".format(field_type.value, field_name))


def _calculate_categorical_training_data(
    field_name: str, field_data: Series, field_type: FieldType
) -> FieldTrainingData:
    valid_values = field_data[field_data.notnull()]
    if field_type == field_type.BOOLEAN:
        valid_values = valid_values.astype("bool")

    bins, counts = np.unique(valid_values, return_counts=True)

    # Note: There is a possible edge case here in which a user passes an infinite value
    # as one of the categories. We chose not to count those values at the moment, since
    # most numpy functions don't handle str and bool well, which would force us to split
    # this function up for each field type.
    return FieldTrainingData(
        field_name=field_name,
        bins=bins.tolist(),
        counts=counts.tolist(),
        num_samples=len(valid_values),
        num_missing_values=len(field_data) - len(valid_values),
        num_unique_values=len(bins),
    )


def _calculate_numeric_training_data(field_name: str, field_data: Series) -> FieldTrainingData:
    # Cast everything to float - this converts None to np.nan
    field_data = field_data.astype(float)

    # Filter out infinite and NaN values (isfinite is False for NaN)
    finite_values = field_data[np.isfinite(field_data)]

    bin_edges = np.histogram_bin_edges(finite_values, bins="auto")
    if len(bin_edges) > MAX_BINS:
        counts, bins = np.histogram(finite_values, bins=MAX_BINS)
    else:
        counts, bins = np.histogram(finite_values, bins=bin_edges)

    return FieldTrainingData(
        field_name=field_name,
        bins=bins.tolist(),
        counts=counts.tolist(),
        min=np.min(finite_values),
        max=np.max(finite_values),
        sum=np.sum(finite_values),  # type: ignore
        median=np.median(finite_values),
        average=np.average(finite_values),
        std=np.std(finite_values),  # type: ignore
        variance=np.var(finite_values),  # type: ignore
        num_samples=len(finite_values),
        num_missing_values=np.count_nonzero(np.isnan(field_data)),
        num_posinf_values=np.count_nonzero(np.isposinf(field_data)),
        num_neginf_values=np.count_nonzero(np.isneginf(field_data)),
        num_zero_values=np.count_nonzero(finite_values == 0),
    )



from typing import Any, Optional

from aporia.core.errors import AporiaError
from aporia.inference.inference_model import InferenceModel

try:
    import numpy
    import pandas

    import aporia.training

    ndarray = numpy.ndarray
    DataFrame = pandas.DataFrame
    TRAINING_ENABLED = True

except ImportError:
    ndarray = Any  # type: ignore
    DataFrame = Any
    TRAINING_ENABLED = False


class Model(InferenceModel):
    """Model object for logging model events."""

    def __init__(self, model_id: str, model_version: str):
        """Initializes an model object.

        Args:
            model_id: Model identifier, as received from the Aporia dashboard.
            model_version: Model version - this can be any string that represents the model
                version, such as "v1" or a git commit hash.
        """
        super().__init__(model_id=model_id, model_version=model_version)

        self._training = None
        if TRAINING_ENABLED:
            self._training = aporia.training.TrainingModel(
                model_id=model_id, model_version=model_version
            )

    def log_training_set(
        self, features: DataFrame, labels: DataFrame, raw_inputs: Optional[DataFrame] = None  # type: ignore
    ):
        """Logs training data.

        Args:
            features: Training set features
            labels: Training set labels
            raw_inputs: Training set raw inputs. Defaults to None.

        Notes:
            * Each dataframe corresponds to a field category defined in create_model_version:
                * features -> features
                * labels -> predictions
                * raw_inputs -> raw_inputs
            * Each column in the dataframe should match a field defined in create_model_version
                * Missing fields will be handled as missing values
                * Columns that do not match a defined field will be ignored
                * The column name must match the field name
            * This function is blocking and may take a while to finish running.
        """
        with self._handle_error("{}"):
            if self._training is None:
                raise AporiaError(
                    short_message="Logging training data failed, Aporia training extension not found",
                    verbose_message=(
                        "The Aporia training extension is required to log training data. "
                        "Install it with `pip install aporia[training] and try again."
                    ),
                )

            self._training.log_training_set(features=features, labels=labels, raw_inputs=raw_inputs)

    def log_test_set(
        self,
        features: DataFrame,  # type: ignore
        predictions: DataFrame,  # type: ignore
        labels: DataFrame,  # type: ignore
        raw_inputs: Optional[DataFrame] = None,  # type: ignore
        confidences: Optional[ndarray] = None,  # type: ignore
    ):
        """Logs test data.

        Args:
            features: Test set features
            predictions: Test set predictions
            labels: Test set labels
            raw_inputs: Test set raw inputs. Defaults to None.
            confidences: Confidence values for the test predictions. Defaults to None.

        Notes:
            * Each dataframe corresponds to a field category defined in create_model_version:
                * features -> features
                * predictions -> predictions
                * labels -> predictions
                * raw_inputs -> raw_inputs
            * Each column in the dataframe should match a field defined in create_model_version
                * Missing fields will be handled as missing values
                * Columns that do not match a defined field will be ignored
                * The column name must match the field name
            * This function is blocking and may take a while to finish running.
        """
        with self._handle_error("{}"):
            if self._training is None:
                raise AporiaError(
                    short_message="Logging test data failed, Aporia training extension not found",
                    verbose_message=(
                        "The Aporia training extension is required to log test data. "
                        "Install it with `pip install aporia[training] and try again."
                    ),
                )

            self._training.log_test_set(
                features=features,
                predictions=predictions,
                labels=labels,
                raw_inputs=raw_inputs,
                confidences=confidences,
            )



import atexit
from contextlib import suppress
import logging
from typing import Dict, Optional

from aporia.model import Model
from .api.create_model_version import (
    run_create_model_version_query,
    validate_fields_input,
)
from .config import Config
from .context import get_context, init_context, reset_context
from .errors import AporiaError, handle_error
from .event_loop import EventLoop
from .graphql_client import GraphQLClient
from .logging_utils import init_logger, LOGGER_NAME, set_log_level

logger = logging.getLogger(LOGGER_NAME)


def init(
    token: Optional[str] = None,
    host: Optional[str] = None,
    environment: Optional[str] = None,
    port: Optional[int] = None,
    verbose: Optional[bool] = None,
    throw_errors: Optional[bool] = None,
    debug: Optional[bool] = None,
):
    """Initializes the Aporia SDK.

    Args:
        token: Authentication token.
        host: Controller host.
        environment: Environment in which aporia is initialized (e.g production, staging).
        port: Controller port. Defaults to 443.
        verbose: True to enable verbose error messages. Defaults to False
        throw_errors: True to cause errors to be raised as exceptions. Defaults to False.
        debug: True to enable debug logs and stack traces in log messages. Defaults to False.

    Notes:
        * The token, host and environment parameters are required.
        * All of the parameters here can also be defined as environment variables:
            * token -> APORIA_TOKEN
            * host -> APORIA_HOST
            * environment -> APORIA_ENVIRONMENT
            * port -> APORIA_PORT
            * verbose -> APORIA_VERBOSE
            * throw_errors -> APORIA_THROW_ERRORS
            * debug -> APORIA_DEBUG
        * Values passed as parameters to aporia.init() override the values from
          the corresponding environment variables.
    """
    init_logger()

    logger.debug("Initializing Aporia SDK.")

    try:
        config = Config(
            token=token,
            host=host,
            environment=environment,
            port=port,
            verbose=verbose,
            throw_errors=throw_errors,
            debug=debug,
        )

        set_log_level(config.debug)

        event_loop = EventLoop()
        graphql_client = GraphQLClient(token=config.token, host=config.host, port=config.port)

        init_context(graphql_client=graphql_client, event_loop=event_loop, config=config)

        atexit.register(shutdown)
        logger.debug("Aporia SDK initialized.")
    except Exception as err:
        handle_error(
            message_format="Initializing Aporia SDK failed, error: {}",
            verbose=False if verbose is None else verbose,
            throw_errors=False if throw_errors is None else throw_errors,
            debug=False if debug is None else debug,
            original_exception=err,
        )


def shutdown():
    """Shuts down the Aporia SDK.

    Notes:
        * It is advised to call flush() before calling shutdown(), to ensure that
          all of the data that was sent reaches the controller.
    """
    logger.debug("Shutting down Aporia SDK.")
    with suppress(Exception):
        reset_context()


def create_model_version(
    model_id: str,
    model_version: str,
    model_type: str,
    features: Dict[str, str],
    predictions: Dict[str, str],
    raw_inputs: Optional[Dict[str, str]] = None,
    metrics: Optional[Dict[str, str]] = None,
) -> Optional[Model]:
    """Create a new model version, and defines a schema for it.

    Args:
        model_id (str): Model identifier, as received from the Aporia dashboard.
        model_version (str): Model version - this can be any string that represents the model
            version, such as "v1" or a git commit hash.
        model_type (str): Model type (also known as objective). The supported model types are:
            * "regression" - for regression models
            * "binary" - for binary classification models
            * "multiclass" - for multiclass classification models
        features (Dict[str, str]): Schema for model features (See notes)
        predictions (Dict[str, str]): Schema for prediction results (See notes)
        raw_inputs (Dict[str, str], optional): Schema for raw inputs (See notes). Defaults to None.
        metrics (Dict[str, str], optional): Schema for prediction metrics (See notes). Defaults to None.

    Notes:
        * A schema is a dict, in which the keys are the fields you wish to report, and the
          values are the types of those fields. For example:
            {
                "feature1": "numeric",
                "feature2": "datetime"
            }
        * The valid field types (and corresponding python types) are:
            | Field Type    | Python Types
            | ------------- | ------------
            | "numeric"     | float, int
            | "categorical" | int
            | "boolean"     | bool
            | "string"      | str
            | "datetime"    | datetime.datetime, or str representing a datetime in ISO-8601 format

    Returns:
        Model object for the new version.
    """
    context = get_context()

    try:
        if len(model_id) == 0 or len(model_version) == 0:
            raise AporiaError("model_id and model_version must be non-empty strings.")

        validate_fields_input(
            features=features, predictions=predictions, raw_inputs=raw_inputs, metrics=metrics,
        )

        context.event_loop.run_coroutine(
            run_create_model_version_query(
                graphql_client=context.graphql_client,
                model_id=model_id,
                model_version=model_version,
                model_type=model_type,
                features=features,
                predictions=predictions,
                raw_inputs=raw_inputs,
                metrics=metrics,
            )
        )

        return Model(model_id=model_id, model_version=model_version)
    except Exception as err:
        handle_error(
            message_format="Creating model version failed, error: {}",
            verbose=context.config.verbose,
            throw_errors=context.config.throw_errors,
            debug=context.config.debug,
            original_exception=err,
        )

        return None



from typing import Any


def orjson_serialize_default_handler(obj: Any) -> str:
    """Default callback for orjson dumps.

    Supports pandas Timestamp type.

    Args:
        obj: Object to serialize

    Returns:
        Serialized object
    """
    # This supports serializing pandas the Timestamp type
    if hasattr(obj, "isoformat"):
        return obj.isoformat()

    # As recommended by orjson, we raise TypeError to indicate this
    # function can't serialize the object
    raise TypeError()



from functools import lru_cache
import logging
from typing import Dict, List, Optional, Tuple

from numpy import ndarray
from pandas import DataFrame

from aporia.core.base_model import BaseModel, validate_model_ready
from aporia.core.errors import AporiaError
from aporia.core.logging_utils import LOGGER_NAME
from aporia.core.types.field import FieldCategory, FieldType
from aporia.pandas.pandas_utils import convert_dtype_to_field_type
from .api.get_model_version import get_model_version
from .api.log_training import (
    calculate_training_data,
    FieldTrainingData,
    log_test_data,
    log_training_data,
)

TRAINING_DATASET_NAME = "training"
TEST_DATASET_NAME = "test"


logger = logging.getLogger(LOGGER_NAME)


class TrainingModel(BaseModel):
    """Model object for logging training events."""

    def __init__(self, model_id: str, model_version: str):
        """Initializes a TrainingModel object.

        Args:
            model_id (str): Model identifier, as received from the Aporia dashboard.
            model_version (str): Model version - this can be any string that represents the model
                version, such as "v1" or a git commit hash.
        """
        super().__init__(model_id=model_id, model_version=model_version, set_ready_when_done=True)

    @validate_model_ready
    def log_training_set(
        self, features: DataFrame, labels: DataFrame, raw_inputs: Optional[DataFrame] = None
    ):
        """See aporia.model.Model."""
        logger.debug("Logging training set.")
        with self._handle_error("Logging training set failed, error: {}"):
            version_schema = self._get_version_schema()
            self._validate_training_dataframes(
                dataset_name="training",
                dataframes={
                    ("features", FieldCategory.FEATURES): features,
                    ("labels", FieldCategory.PREDICTIONS): labels,
                    ("raw inputs", FieldCategory.RAW_INPUTS): raw_inputs,
                },
                schema=version_schema,
            )

            raw_inputs_training_data = None
            if raw_inputs is not None:
                raw_inputs_training_data = self._calculate_dataframe_training_data(
                    raw_inputs, version_schema[FieldCategory.RAW_INPUTS]
                )

            self._event_loop.run_coroutine(
                log_training_data(
                    graphql_client=self._graphql_client,
                    model_id=self.model_id,
                    model_version=self.model_version,
                    features=self._calculate_dataframe_training_data(
                        features, version_schema[FieldCategory.FEATURES]
                    ),
                    labels=self._calculate_dataframe_training_data(
                        labels, version_schema[FieldCategory.PREDICTIONS]
                    ),
                    raw_inputs=raw_inputs_training_data,
                )
            )

    @validate_model_ready
    def log_test_set(
        self,
        features: DataFrame,
        predictions: DataFrame,
        labels: DataFrame,
        raw_inputs: Optional[DataFrame] = None,
        confidences: Optional[ndarray] = None,
    ):
        """See aporia.model.Model."""
        logger.debug("Logging test set.")
        with self._handle_error("Logging test set failed, error: {}"):
            version_schema = self._get_version_schema()
            self._validate_training_dataframes(
                dataset_name="test",
                dataframes={
                    ("features", FieldCategory.FEATURES): features,
                    ("predictions", FieldCategory.PREDICTIONS): predictions,
                    ("labels", FieldCategory.PREDICTIONS): labels,
                    ("raw inputs", FieldCategory.RAW_INPUTS): raw_inputs,
                },
                schema=version_schema,
            )

            raw_inputs_training_data = None
            if raw_inputs is not None:
                raw_inputs_training_data = self._calculate_dataframe_training_data(
                    raw_inputs, version_schema[FieldCategory.RAW_INPUTS]
                )

            self._event_loop.run_coroutine(
                log_test_data(
                    graphql_client=self._graphql_client,
                    model_id=self.model_id,
                    model_version=self.model_version,
                    features=self._calculate_dataframe_training_data(
                        features, version_schema[FieldCategory.FEATURES]
                    ),
                    predictions=self._calculate_dataframe_training_data(
                        predictions, version_schema[FieldCategory.PREDICTIONS]
                    ),
                    labels=self._calculate_dataframe_training_data(
                        labels, version_schema[FieldCategory.PREDICTIONS]
                    ),
                    raw_inputs=raw_inputs_training_data,
                )
            )

    @lru_cache(maxsize=1)
    def _get_version_schema(self) -> Dict[FieldCategory, Dict[str, FieldType]]:
        return self._event_loop.run_coroutine(
            get_model_version(
                graphql_client=self._graphql_client,
                model_id=self.model_id,
                model_version=self.model_version,
            )
        )

    @staticmethod
    def _validate_training_dataframes(
        dataset_name: str,
        dataframes: Dict[Tuple[str, FieldCategory], Optional[DataFrame]],
        schema: Dict[FieldCategory, Dict[str, FieldType]],
    ):
        for (training_data_type, field_category), data in dataframes.items():
            if data is None:
                continue

            # Check that a schema is defined for this category
            if field_category not in schema:
                raise AporiaError(
                    "Cannot report {} set {}, because it was not defined during model "
                    "version creation.".format(dataset_name, training_data_type)
                )

            # Check for missing fields
            column_names = set(data.columns)
            missing_fields = schema[field_category].keys() - column_names
            if len(missing_fields) > 0:
                logger.warning(
                    "Detected missing {training_type} in the {dataset} data - the following "
                    "{training_type} were defined during model version creation but are "
                    "not present in the {dataset} data: {missing_fields}".format(
                        training_type=training_data_type,
                        dataset=dataset_name,
                        missing_fields=missing_fields,
                    )
                )

            # Check for undefined fields
            undefined_fields = column_names - schema[field_category].keys()
            if len(undefined_fields) > 0:
                logger.warning(
                    "Detected undefined {training_type} in the {dataset} data - the following "
                    "{training_type} were found in the {dataset} data, but were not defined "
                    "during model version creation: {undefined_fields}".format(
                        training_type=training_data_type,
                        dataset=dataset_name,
                        undefined_fields=undefined_fields,
                    )
                )

            for field_name, field_type in schema[field_category].items():
                if field_name in missing_fields:
                    continue

                column_data_without_nulls = data[field_name].dropna().infer_objects()
                inferred_field_type = convert_dtype_to_field_type(
                    column_data_without_nulls.dtype  # type: ignore
                )

                # Don't verify boolean fields, since we're going to cast them anyway
                if field_type != FieldType.BOOLEAN and field_type != inferred_field_type:
                    logger.warning(
                        "Detected type mismatch on field {field_name}: expected {expected_type} "
                        "type, got {actual_type}".format(
                            field_name=field_name,
                            expected_type=field_type.value,
                            actual_type=column_data_without_nulls.dtype,
                        )
                    )

    @staticmethod
    def _calculate_dataframe_training_data(
        data: DataFrame, fields_schema: Dict[str, FieldType]
    ) -> List[FieldTrainingData]:
        training_data = []
        for field_name, field_data in data.items():
            # Ignore fields that are not defined in the model version schema
            if field_name not in fields_schema:
                continue

            training_data.append(
                calculate_training_data(
                    field_name=field_name,
                    field_data=field_data,
                    field_type=fields_schema[field_name],
                )
            )

        return training_data



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['aporia',
 'aporia.core',
 'aporia.core.api',
 'aporia.core.types',
 'aporia.inference',
 'aporia.inference.api',
 'aporia.pandas',
 'aporia.training',
 'aporia.training.api']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.6.2,<4.0.0',
 'certifi>=2020.12.5,<2021.0.0',
 'orjson>=2.0.4,<4.0.0',
 'tenacity>=6.2.0,<7.0.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=1.5.0,<2.0.0'],
 'all': ['numpy>=1.15.0,<2.0.0', 'pandas>=0.21,<2.0.0'],
 'pandas': ['numpy>=1.15.0,<2.0.0', 'pandas>=0.21,<2.0.0'],
 'training': ['numpy>=1.15.0,<2.0.0', 'pandas>=0.21,<2.0.0']}

setup_kwargs = {
    'name': 'aporia',
    'version': '1.0.56',
    'description': 'Aporia SDK',
    'long_description': '# Aporia SDK',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aporia-ai/sdk',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.5.3,<4.0.0',
}


setup(**setup_kwargs)
